#!/usr/bin/env ruby
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version. This program is distributed in the hope that
# it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
# the GNU General Public License for more details. You should have
# received a copy of the GNU General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA 02111-1307, USA.
#
# All the code to drive the tray icon and the popup menu
#

require 'gtk2'
require 'gconf2'
require 'gtktrayicon'
require 'GMailConnection.rb'
require 'PreferencesDialog.rb'
require 'AboutDialog.rb'

class TrayIcon
 
 private 

 def connect
	username = $gconf[@username_key]
	password = $gconf[@password_key]

	if username == nil or password == nil then
		self.tooltip = "Not connected"
		@gmail_conn = nil
	else
		@icon.set("gmail-connecting.png")

		begin
			self.tooltip = "Connecting.."

			@gmail_conn = GMailConnection.new(username, password)
			@check_failures = 0

		rescue
			self.tooltip = "Login failure"
			@error_dialog.show_all
			@gmail_conn = nil
		end
	end

	@need_reconnect = false
 end

 def setup_menu
	menu = Gtk::Menu.new

	item = Gtk::ImageMenuItem.new("Check Now")
	item.image = Gtk::Image.new(Gtk::Stock::REFRESH, Gtk::IconSize::MENU)
	item.signal_connect('activate') do
		@force_refresh = true
	end
	menu.append(item)

	item = Gtk::ImageMenuItem.new(Gtk::Stock::PREFERENCES)
	item.signal_connect('activate') do
		@pref_dialog.display
	end
	menu.append(item)

	item = Gtk::MenuItem.new("About")
	item.signal_connect('activate') do
		@about_dialog = AboutDialog.new if @about_dialog == nil
		@about_dialog.display
	end
	menu.append(item)

	item = Gtk::ImageMenuItem.new(Gtk::Stock::QUIT)
	item.signal_connect('activate') do
		Gtk::main_quit
	end
	menu.append(item)

	menu.show_all

	@event_box.signal_connect('button_press_event') do |widget, event|
		if event.kind_of? Gdk::EventButton and event.button == 3
			menu.popup(nil, nil, event.button, event.time)
		end
		if event.kind_of? Gdk::EventButton and event.button == 1
			my_p = IO.popen("gnome-open http://gmail.google.com/gmail")
			my_p.close
		end
	end
 end

 def error_dialog
	@error_dialog = Gtk::Dialog.new
	@error_dialog.title = "Connection Error"

	hbox = Gtk::HBox.new
	hbox.border_width = 10
	hbox.pack_start(Gtk::Image.new(Gtk::Stock::DIALOG_ERROR, Gtk::IconSize::DIALOG))

	label = Gtk::Label.new("Unable to log in to GMail. " +
			       "Check the account settings are correct and " +
			       "try again.")
	label.wrap = true
	hbox.pack_start(label, false, false, 10)

	@error_dialog.vbox.pack_start(hbox)

	ok = Gtk::Button.new(Gtk::Stock::OK)
	ok.signal_connect('clicked') do
		@error_dialog.hide
		@pref_dialog.display
	end

	@error_dialog.add_action_widget(ok, Gtk::Dialog::RESPONSE_NONE)
 end

 public
 
 def initialize(gconf_root)
 	@gconf_root = gconf_root

	@pref_dialog = PreferencesDialog.new(@gconf_root) 

 	@tray = Gtk::TrayIcon.new("GMail")
	@event_box = Gtk::EventBox.new
	@icon = Gtk::Image.new("gmail-connecting.png")
	@tooltips = Gtk::Tooltips.new
	@tooltips.enable

	@tray.add(@event_box)
	@event_box.add(@icon)
	@tray.show_all

	setup_menu

	# gconf keys

 	@username_key = "#{@gconf_root}/username"
 	@password_key = "#{@gconf_root}/password"
	@update_freq_key = "#{@gconf_root}/update_time"
	@non_inbox_key = "#{@gconf_root}/non_inbox"

	# first time we run, pop up a "login" box (preferences dialog)

	if $gconf[@username_key] == nil then
		@pref_dialog.display
	end

	# if the keys change, we need to reconnect

	$gconf.add_dir(@gconf_root)
	$gconf.notify_add(@username_key) { @need_reconnect = true }
	$gconf.notify_add(@password_key) { @need_reconnect = true }
	$gconf.notify_add(@update_freq_key) { @force_refresh = true }
	$gconf.notify_add(@non_inbox_key) { @force_refresh = true }

	# set up thread

	Thread.new do
		begin
			thread_func
		rescue 
		end

		puts "Thread died (this should never happen)"
	end

	# build the error dialog for later

	error_dialog
 end

 def thread_func
	connect

 	loop do
		check_mail
		sleep_time = $gconf[@update_freq_key]

		sleep_time = 300 if sleep_time == nil

		# sleep in 1 second intervals so we can get 
		# an "immediate" response if we need to reconnect

		@force_refresh = false

		for i in 0..sleep_time-1 do
			sleep 1

			if @need_reconnect then

				connect
				
				break
			end
			if @force_refresh then
				# if we are forcing a refresh, but we
				# had a login failure, make the refresh
				# trigger another login attempt

				if @gmail_conn == nil then
					connect
				end
				break
			end
		end
	end
 end

 def tooltip=(s)

	@tooltips.set_tip(@tray, s, s)

 end

 def check_mail

	if @gmail_conn != nil then
	 	# check mail and update the tooltip

		begin
			non_inbox = if $gconf[@non_inbox_key] != nil then
						$gconf[@non_inbox_key]
					else
						true
					end
			new_messages = @gmail_conn.check_mail(non_inbox)
		rescue
			# error checking mail
			# change the icon so we can see it is broken

			self.tooltip = "Error while checking mail"
			@icon.set("gmail-connecting.png")

			# when we reach 3 consecutive check failures,
			# try reconnecting

			@check_failures += 1

			if @check_failures >= 3 then
				@need_reconnect = true
				@gmail_conn = nil
			end

			return
		end

		# reset the failure count if we get a successful check

		@check_failures = 0

		message = case new_messages
			when 0 then "No new messages"
			when 1 then "1 new message"
			else "#{new_messages} new messages"
		end

		if new_messages > 0 then
			@icon.set("gmail.png")
		else
			@icon.set("gmail-idle.png")
		end

		self.tooltip = message
	end
 end
end

